@@ -54,6 +54,9 @@ struct _BraseroVideoOptionsPrivate
 	GtkWidget *vcd_button;
 	GtkWidget *svcd_button;
 
+	GtkWidget *button_native;
+	GtkWidget *button_ntsc;
+	GtkWidget *button_pal;
 	GtkWidget *button_4_3;
 	GtkWidget *button_16_9;
 };
@@ -68,41 +71,84 @@ enum {
 G_DEFINE_TYPE (BraseroVideoOptions, brasero_video_options, GTK_TYPE_ALIGNMENT);
 
 static void
-brasero_video_options_audio_AC3 (BraseroVideoOptions *options)
+brasero_video_options_update_from_tag (BraseroVideoOptions *options,
+                                       const gchar *tag)
 {
-	GValue *value = NULL;
 	BraseroVideoOptionsPrivate *priv;
 
-	priv = BRASERO_VIDEO_OPTIONS_PRIVATE (options);
-
-	if (!priv->session)
+	if (!tag)
 		return;
 
-	value = g_new0 (GValue, 1);
-	g_value_init (value, G_TYPE_INT);
-	g_value_set_int (value, BRASERO_AUDIO_FORMAT_AC3);
-	brasero_burn_session_tag_add (priv->session,
-				      BRASERO_DVD_STREAM_FORMAT,
-				      value);
-}
-
-static void
-brasero_video_options_audio_MP2 (BraseroVideoOptions *options)
-{
-	GValue *value = NULL;
-	BraseroVideoOptionsPrivate *priv;
-
 	priv = BRASERO_VIDEO_OPTIONS_PRIVATE (options);
+	
+	if (!strcmp (tag, BRASERO_VCD_TYPE)) {
+		BraseroMedia media;
+		gint svcd_type;
 
-	if (!priv->session)
-		return;
+		media = brasero_burn_session_get_dest_media (priv->session);
+		if (media & BRASERO_MEDIUM_DVD) {
+			/* Don't change anything in this case
+			 * as the tag has no influence over 
+			 * this type of image */
+			return;
+		}
+		else if (media & BRASERO_MEDIUM_FILE) {
+			BraseroImageFormat format;
 
-	value = g_new0 (GValue, 1);
-	g_value_init (value, G_TYPE_INT);
-	g_value_set_int (value, BRASERO_AUDIO_FORMAT_MP2);
-	brasero_burn_session_tag_add (priv->session,
-				      BRASERO_DVD_STREAM_FORMAT,
-				      value);
+			format = brasero_burn_session_get_output_format (priv->session);
+
+			/* Same as above for the following case */
+			if (format == BRASERO_IMAGE_FORMAT_BIN)
+				return;
+		}
+
+		svcd_type = brasero_burn_session_tag_lookup_int (priv->session, tag);
+		if (svcd_type == BRASERO_SVCD) {
+			if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->svcd_button)))
+				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->svcd_button), TRUE);
+
+				gtk_widget_set_sensitive (priv->button_4_3, TRUE);
+				gtk_widget_set_sensitive (priv->button_16_9, TRUE);
+		}
+		else {
+			if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->vcd_button)))
+				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->vcd_button), TRUE);
+
+			gtk_widget_set_sensitive (priv->button_4_3, FALSE);
+			gtk_widget_set_sensitive (priv->button_16_9, FALSE);
+		}
+	}
+	else if (!strcmp (tag, BRASERO_VIDEO_OUTPUT_FRAMERATE)) {
+		GValue *value = NULL;
+
+		brasero_burn_session_tag_lookup (priv->session,
+						 tag,
+						 &value);
+		if (value) {
+			if (g_value_get_int (value) == BRASERO_VIDEO_FRAMERATE_NTSC) {
+				if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->button_ntsc)))
+					gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->button_ntsc), TRUE);
+			}
+			else {
+				if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->button_pal)))
+					gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->button_pal), TRUE);
+			}
+		}
+		else if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->button_native)))
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->button_native), TRUE);
+	}
+	else if (!strcmp (tag, BRASERO_VIDEO_OUTPUT_ASPECT)) {
+		gint aspect_type = brasero_burn_session_tag_lookup_int (priv->session, tag);
+
+		if (aspect_type == BRASERO_VIDEO_ASPECT_16_9) {
+			if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->button_16_9)))
+				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->button_16_9), TRUE);
+		}
+		else {
+			if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->button_4_3)))
+				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->button_4_3), TRUE);
+		}
+	}
 }
 
 static void
@@ -119,7 +165,6 @@ brasero_video_options_update (BraseroVideoOptions *options)
 
 	media = brasero_burn_session_get_dest_media (priv->session);
 	if (media & BRASERO_MEDIUM_DVD) {
-		brasero_video_options_audio_AC3 (options);
 		gtk_widget_hide (priv->vcd_label);
 		gtk_widget_hide (priv->vcd_button);
 		gtk_widget_hide (priv->svcd_button);
@@ -128,74 +173,46 @@ brasero_video_options_update (BraseroVideoOptions *options)
 		gtk_widget_set_sensitive (priv->button_16_9, TRUE);
 	}
 	else if (media & BRASERO_MEDIUM_CD) {
-		brasero_video_options_audio_MP2 (options);
 		gtk_widget_show (priv->vcd_label);
 		gtk_widget_show (priv->vcd_button);
 		gtk_widget_show (priv->svcd_button);
 
-		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->vcd_button))) {
-			gtk_widget_set_sensitive (priv->button_4_3, FALSE);
-			gtk_widget_set_sensitive (priv->button_16_9, FALSE);
-		}
-		else {
-			gtk_widget_set_sensitive (priv->button_4_3, TRUE);
-			gtk_widget_set_sensitive (priv->button_16_9, TRUE);
-		}
+		brasero_video_options_update_from_tag (options, BRASERO_VCD_TYPE);
 	}
 	else if (media & BRASERO_MEDIUM_FILE) {
 		BraseroImageFormat format;
 
-		/* if we create a CUE file then that's a (S)VCD */
+		/* Hide any options about (S)VCD type
+		 * as this is handled in BraseroImageTypeChooser 
+		 * object */
+		gtk_widget_hide (priv->vcd_label);
+		gtk_widget_hide (priv->vcd_button);
+		gtk_widget_hide (priv->svcd_button);
+
 		format = brasero_burn_session_get_output_format (priv->session);
-		if (format == BRASERO_IMAGE_FORMAT_NONE)
-			return;
-
-		if (format == BRASERO_IMAGE_FORMAT_CUE) {
-			brasero_video_options_audio_MP2 (options);
-			gtk_widget_show (priv->vcd_label);
-			gtk_widget_show (priv->vcd_button);
-			gtk_widget_show (priv->svcd_button);
-
-			if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->vcd_button))) {
-				gtk_widget_set_sensitive (priv->button_4_3, FALSE);
-				gtk_widget_set_sensitive (priv->button_16_9, FALSE);
-			}
-			else {
-				gtk_widget_set_sensitive (priv->button_4_3, TRUE);
-				gtk_widget_set_sensitive (priv->button_16_9, TRUE);
-			}
-		}
-		else if (format == BRASERO_IMAGE_FORMAT_BIN) {
-			brasero_video_options_audio_AC3 (options);
-			gtk_widget_hide (priv->vcd_label);
-			gtk_widget_hide (priv->vcd_button);
-			gtk_widget_hide (priv->svcd_button);
-
+		if (format == BRASERO_IMAGE_FORMAT_BIN) {
 			gtk_widget_set_sensitive (priv->button_4_3, TRUE);
 			gtk_widget_set_sensitive (priv->button_16_9, TRUE);
 		}
+		else if (format == BRASERO_IMAGE_FORMAT_CUE)
+			brasero_video_options_update_from_tag (options, BRASERO_VCD_TYPE);
 	}
 }
 
 static void
-brasero_video_options_set_tag (BraseroVideoOptions *options,
-			       const gchar *tag,
-			       gint contents)
+brasero_video_options_output_changed_cb (BraseroBurnSession *session,
+                                         BraseroMedium *former_medium,
+                                         BraseroVideoOptions *options)
 {
-	GValue *value;
-	BraseroVideoOptionsPrivate *priv;
+	brasero_video_options_update (options);
+}
 
-	priv = BRASERO_VIDEO_OPTIONS_PRIVATE (options);
-
-	if (!priv->session)
-		return;
-
-	value = g_new0 (GValue, 1);
-	g_value_init (value, G_TYPE_INT);
-	g_value_set_int (value, contents);
-	brasero_burn_session_tag_add (priv->session,
-				      tag,
-				      value);
+static void
+brasero_video_options_tag_changed_cb (BraseroBurnSession *session,
+                                      const gchar *tag,
+                                      BraseroVideoOptions *options)
+{
+	brasero_video_options_update_from_tag (options, tag);
 }
 
 static void
@@ -207,11 +224,13 @@ brasero_video_options_SVCD (GtkToggleButton *button,
 	if (!gtk_toggle_button_get_active (button))
 		return;
 
-	brasero_video_options_set_tag (options,
-				       BRASERO_VCD_TYPE,
-				       BRASERO_SVCD);
-
 	priv = BRASERO_VIDEO_OPTIONS_PRIVATE (options);
+	brasero_burn_session_tag_add_int (priv->session,
+	                                  BRASERO_VCD_TYPE,
+	                                  BRASERO_SVCD);
+
+	/* NOTE: this is only possible when that's
+	 * not an image */
 
 	gtk_widget_set_sensitive (priv->button_4_3, TRUE);
 	gtk_widget_set_sensitive (priv->button_16_9, TRUE);
@@ -226,11 +245,13 @@ brasero_video_options_VCD (GtkToggleButton *button,
 	if (!gtk_toggle_button_get_active (button))
 		return;
 
-	brasero_video_options_set_tag (options,
-				       BRASERO_VCD_TYPE,
-				       BRASERO_VCD_V2);
-
 	priv = BRASERO_VIDEO_OPTIONS_PRIVATE (options);
+	brasero_burn_session_tag_add_int (priv->session,
+	                                  BRASERO_VCD_TYPE,
+	                                  BRASERO_VCD_V2);
+
+	/* NOTE: this is only possible when that's
+	 * not an image */
 	gtk_widget_set_sensitive (priv->button_4_3, FALSE);
 	gtk_widget_set_sensitive (priv->button_16_9, FALSE);
 
@@ -241,24 +262,30 @@ static void
 brasero_video_options_NTSC (GtkToggleButton *button,
 			    BraseroVideoOptions *options)
 {
+	BraseroVideoOptionsPrivate *priv;
+
 	if (!gtk_toggle_button_get_active (button))
 		return;
 
-	brasero_video_options_set_tag (options,
-				       BRASERO_VIDEO_OUTPUT_FRAMERATE,
-				       BRASERO_VIDEO_FRAMERATE_NTSC);
+	priv = BRASERO_VIDEO_OPTIONS_PRIVATE (options);
+	brasero_burn_session_tag_add_int (priv->session,
+	                                  BRASERO_VIDEO_OUTPUT_FRAMERATE,
+	                                  BRASERO_VIDEO_FRAMERATE_NTSC);
 }
 
 static void
 brasero_video_options_PAL_SECAM (GtkToggleButton *button,
 				 BraseroVideoOptions *options)
 {
+	BraseroVideoOptionsPrivate *priv;
+
 	if (!gtk_toggle_button_get_active (button))
 		return;
 
-	brasero_video_options_set_tag (options,
-				       BRASERO_VIDEO_OUTPUT_FRAMERATE,
-				       BRASERO_VIDEO_FRAMERATE_PAL_SECAM);
+	priv = BRASERO_VIDEO_OPTIONS_PRIVATE (options);
+	brasero_burn_session_tag_add_int (priv->session,
+	                                  BRASERO_VIDEO_OUTPUT_FRAMERATE,
+	                                  BRASERO_VIDEO_FRAMERATE_PAL_SECAM);
 }
 
 static void
@@ -267,10 +294,10 @@ brasero_video_options_native_framerate (GtkToggleButton *button,
 {
 	BraseroVideoOptionsPrivate *priv;
 
-	priv = BRASERO_VIDEO_OPTIONS_PRIVATE (options);
 	if (!gtk_toggle_button_get_active (button))
 		return;
 
+	priv = BRASERO_VIDEO_OPTIONS_PRIVATE (options);
 	brasero_burn_session_tag_remove (priv->session, BRASERO_VIDEO_OUTPUT_FRAMERATE);
 }
 
@@ -278,24 +305,30 @@ static void
 brasero_video_options_16_9 (GtkToggleButton *button,
 			    BraseroVideoOptions *options)
 {
+	BraseroVideoOptionsPrivate *priv;
+
 	if (!gtk_toggle_button_get_active (button))
 		return;
 
-	brasero_video_options_set_tag (options,
-				       BRASERO_VIDEO_OUTPUT_ASPECT,
-				       BRASERO_VIDEO_ASPECT_16_9);
+	priv = BRASERO_VIDEO_OPTIONS_PRIVATE (options);
+	brasero_burn_session_tag_add_int (priv->session,
+	                                  BRASERO_VIDEO_OUTPUT_ASPECT,
+	                                  BRASERO_VIDEO_ASPECT_16_9);
 }
 
 static void
 brasero_video_options_4_3 (GtkToggleButton *button,
 			   BraseroVideoOptions *options)
 {
+	BraseroVideoOptionsPrivate *priv;
+
 	if (!gtk_toggle_button_get_active (button))
 		return;
 
-	brasero_video_options_set_tag (options,
-				       BRASERO_VIDEO_OUTPUT_ASPECT,
-				       BRASERO_VIDEO_ASPECT_4_3);
+	priv = BRASERO_VIDEO_OPTIONS_PRIVATE (options);
+	brasero_burn_session_tag_add_int (priv->session,
+	                                  BRASERO_VIDEO_OUTPUT_ASPECT,
+	                                  BRASERO_VIDEO_ASPECT_4_3);
 }
 
 void
@@ -306,6 +339,12 @@ brasero_video_options_set_session (BraseroVideoOptions *options,
 
 	priv = BRASERO_VIDEO_OPTIONS_PRIVATE (options);
 	if (priv->session) {
+		g_signal_handlers_disconnect_by_func (priv->session,
+		                                      brasero_video_options_output_changed_cb,
+		                                      options);
+		g_signal_handlers_disconnect_by_func (priv->session,
+		                                      brasero_video_options_tag_changed_cb,
+		                                      options);
 		g_object_unref (priv->session);
 		priv->session = NULL;
 	}
@@ -313,6 +352,17 @@ brasero_video_options_set_session (BraseroVideoOptions *options,
 	if (session) {
 		priv->session = g_object_ref (session);
 		brasero_video_options_update (options);
+		brasero_video_options_update_from_tag (options, BRASERO_VIDEO_OUTPUT_FRAMERATE);
+		brasero_video_options_update_from_tag (options, BRASERO_VIDEO_OUTPUT_ASPECT);
+
+		g_signal_connect (priv->session,
+		                  "output-changed",
+		                  G_CALLBACK (brasero_video_options_output_changed_cb),
+		                  options);
+		g_signal_connect (priv->session,
+		                  "tag-changed",
+		                  G_CALLBACK (brasero_video_options_tag_changed_cb),
+		                  options);
 	}
 }
 
@@ -396,9 +446,9 @@ brasero_video_options_init (BraseroVideoOptions *object)
 			  GTK_FILL,
 			  0, 0);
 
-	button1 = gtk_radio_button_new_with_mnemonic (NULL,
-						      _("_NTSC"));
-	gtk_widget_set_tooltip_text (button1, _("Format used mostly on the North American Continent"));
+	button1 = gtk_radio_button_new_with_mnemonic (NULL, _("_NTSC"));
+	priv->button_ntsc = button1;
+	gtk_widget_set_tooltip_text (button1, _("Format used mostly on the North American continent"));
 	g_signal_connect (button1,
 			  "toggled",
 			  G_CALLBACK (brasero_video_options_NTSC),
@@ -411,8 +461,8 @@ brasero_video_options_init (BraseroVideoOptions *object)
 			  GTK_FILL,
 			  0, 0);
 
-	button2 = gtk_radio_button_new_with_mnemonic_from_widget (GTK_RADIO_BUTTON (button1),
-								  _("_PAL/SECAM"));
+	button2 = gtk_radio_button_new_with_mnemonic_from_widget (GTK_RADIO_BUTTON (button1), _("_PAL/SECAM"));
+	priv->button_pal = button2;
 	gtk_widget_set_tooltip_text (button2, _("Format used mostly in Europe"));
 	g_signal_connect (button2,
 			  "toggled",
@@ -428,6 +478,7 @@ brasero_video_options_init (BraseroVideoOptions *object)
 
 	button3 = gtk_radio_button_new_with_mnemonic_from_widget (GTK_RADIO_BUTTON (button1),
 								  _("Native _format"));
+	priv->button_native = button3;
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button3), TRUE);
 	g_signal_connect (button3,
 			  "toggled",
@@ -496,7 +547,7 @@ brasero_video_options_init (BraseroVideoOptions *object)
 			  GTK_FILL,
 			  0, 0);
 
-	button1 = gtk_radio_button_new_with_mnemonic_from_widget (NULL, _("Create a SVCD"));
+	button1 = gtk_radio_button_new_with_mnemonic_from_widget (NULL, _("Create an SVCD"));
 	priv->svcd_button = button1;
 	gtk_table_attach (GTK_TABLE (table),
 			  button1,
@@ -534,14 +585,6 @@ brasero_video_options_init (BraseroVideoOptions *object)
 
 	gtk_widget_show_all (widget);
 	gtk_container_add (GTK_CONTAINER (object), widget);
-
-	/* Just to make sure our tags are correct in BraseroBurnSession */
-	brasero_video_options_set_tag (object,
-				       BRASERO_VCD_TYPE,
-				       BRASERO_SVCD);
-	brasero_video_options_set_tag (object,
-				       BRASERO_VIDEO_OUTPUT_ASPECT,
-				       BRASERO_VIDEO_ASPECT_4_3);
 }
 
 static void
@@ -551,6 +594,12 @@ brasero_video_options_finalize (GObject *object)
 
 	priv = BRASERO_VIDEO_OPTIONS_PRIVATE (object);
 	if (priv->session) {
+		g_signal_handlers_disconnect_by_func (priv->session,
+		                                      brasero_video_options_output_changed_cb,
+		                                      object);
+		g_signal_handlers_disconnect_by_func (priv->session,
+		                                      brasero_video_options_tag_changed_cb,
+		                                      object);
 		g_object_unref (priv->session);
 		priv->session = NULL;
 	}
