@@ -44,11 +44,11 @@ G_BEGIN_DECLS
 #define LIBBRASERO_MEDIA_VERSION_MINOR						\
 	29
 #define LIBBRASERO_MEDIA_VERSION_MICRO						\
-	3
+	6
 #define LIBBRASERO_MEDIA_AGE							\
 	202
 #define LIBBRASERO_MEDIA_INSTALL							\
-	"/gnome/usr"
+	"/usr/local"
 
 /**
  * To start and stop the library
