@@ -48,7 +48,6 @@
 #include <libburn/libburn.h>
 
 #include "burn-libburnia.h"
-#include "burn-libisofs.h"
 #include "burn-job.h"
 #include "brasero-units.h"
 #include "brasero-plugin-registration.h"
@@ -56,6 +55,14 @@
 #include "brasero-track-data.h"
 #include "brasero-track-image.h"
 
+
+#define BRASERO_TYPE_LIBISOFS         (brasero_libisofs_get_type ())
+#define BRASERO_LIBISOFS(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), BRASERO_TYPE_LIBISOFS, BraseroLibisofs))
+#define BRASERO_LIBISOFS_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST((k), BRASERO_TYPE_LIBISOFS, BraseroLibisofsClass))
+#define BRASERO_IS_LIBISOFS(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), BRASERO_TYPE_LIBISOFS))
+#define BRASERO_IS_LIBISOFS_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), BRASERO_TYPE_LIBISOFS))
+#define BRASERO_LIBISOFS_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), BRASERO_TYPE_LIBISOFS, BraseroLibisofsClass))
+
 BRASERO_PLUGIN_BOILERPLATE (BraseroLibisofs, brasero_libisofs, BRASERO_TYPE_JOB, BraseroJob);
 
 struct _BraseroLibisofsPrivate {
@@ -140,7 +147,7 @@ brasero_libisofs_write_sector_to_fd (BraseroLibisofs *self,
 		gint written;
 
 		written = write (fd,
-				 buffer + bytes_written,
+				 ((gchar *) buffer) + bytes_written,
 				 bytes_remaining);
 
 		if (priv->cancel)
@@ -182,6 +189,8 @@ brasero_libisofs_write_image_to_fd_thread (BraseroLibisofs *self)
 
 	priv = BRASERO_LIBISOFS_PRIVATE (self);
 
+	brasero_job_set_nonblocking (BRASERO_JOB (self), NULL);
+
 	brasero_job_set_current_action (BRASERO_JOB (self),
 					BRASERO_BURN_ACTION_CREATING_IMAGE,
 					NULL,
@@ -316,7 +325,7 @@ brasero_libisofs_create_image (BraseroLibisofs *self,
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("Libisofs could not be initialized."));
+			     _("libisofs could not be initialized."));
 		return BRASERO_BURN_ERR;
 	}
 
@@ -447,19 +456,10 @@ brasero_libisofs_import_last_session (BraseroLibisofs *self,
 
 	priv = BRASERO_LIBISOFS_PRIVATE (self);
 
-	priv->ctx = brasero_libburn_common_ctx_new (BRASERO_JOB (self), error);
+	priv->ctx = brasero_libburn_common_ctx_new (BRASERO_JOB (self), FALSE, error);
 	if (!priv->ctx)
 		return BRASERO_BURN_ERR;
 
-	src = g_new0 (IsoDataSource, 1);
-	src->version = 0;
-	src->refcount = 1;
-	src->read_block = brasero_libisofs_import_read;
-	src->open = brasero_libisofs_import_open;
-	src->close = brasero_libisofs_import_close;
-	src->free_data = brasero_libisofs_import_free;
-	src->data = priv->ctx->drive;
-
 	result = iso_read_opts_new (&opts, 0);
 	if (result < 0) {
 		g_set_error (error,
@@ -469,6 +469,15 @@ brasero_libisofs_import_last_session (BraseroLibisofs *self,
 		return BRASERO_BURN_ERR;
 	}
 
+	src = g_new0 (IsoDataSource, 1);
+	src->version = 0;
+	src->refcount = 1;
+	src->read_block = brasero_libisofs_import_read;
+	src->open = brasero_libisofs_import_open;
+	src->close = brasero_libisofs_import_close;
+	src->free_data = brasero_libisofs_import_free;
+	src->data = priv->ctx->drive;
+
 	brasero_job_get_last_session_address (BRASERO_JOB (self), &session_block);
 	iso_read_opts_set_start_block (opts, session_block);
 
@@ -485,6 +494,7 @@ brasero_libisofs_import_last_session (BraseroLibisofs *self,
 	}
 
 	if (result < 0) {
+		BRASERO_JOB_LOG (self, "Import failed 0x%x", result);
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_IMAGE_LAST_SESSION,
@@ -546,6 +556,31 @@ brasero_libisofs_create_volume_thread (gpointer data)
 		goto end;
 	}
 
+	iso_write_opts_new (&opts, 2);
+	iso_write_opts_set_relaxed_vol_atts(opts, 1);
+
+	brasero_job_get_flags (BRASERO_JOB (self), &flags);
+	if (flags & BRASERO_BURN_FLAG_MERGE) {
+		BraseroBurnResult result;
+
+		result = brasero_libisofs_import_last_session (self,
+							       image,
+							       opts,
+							       &priv->error);
+		if (result != BRASERO_BURN_OK) {
+			g_free (label);
+			goto end;
+		}
+	}
+	else if (flags & BRASERO_BURN_FLAG_APPEND) {
+		goffset start_block;
+
+		brasero_job_get_next_writable_address (BRASERO_JOB (self), &start_block);
+		iso_write_opts_set_ms_block (opts, start_block);
+	}
+
+	/* set label but set it after merging so the
+	 * new does not get replaced by the former */
 	publisher = g_strdup_printf ("Brasero-%i.%i.%i",
 				     BRASERO_MAJOR_VERSION,
 				     BRASERO_MINOR_VERSION,
@@ -560,27 +595,6 @@ brasero_libisofs_create_volume_thread (gpointer data)
 	g_free (publisher);
 	g_free (label);
 
-	iso_write_opts_new (&opts, 2);
-
-	brasero_job_get_flags (BRASERO_JOB (self), &flags);
-	if (flags & BRASERO_BURN_FLAG_MERGE) {
-		BraseroBurnResult result;
-
-		result = brasero_libisofs_import_last_session (self,
-							       image,
-							       opts,
-							       &priv->error);
-		if (result != BRASERO_BURN_OK)
-			goto end;
-	}
-	else if (flags & BRASERO_BURN_FLAG_APPEND) {
-		goffset start_block;
-
-		brasero_job_get_next_writable_address (BRASERO_JOB (self), &start_block);
-		iso_write_opts_set_ms_block (opts, start_block);
-	}
-
-
 	brasero_job_start_progress (BRASERO_JOB (self), FALSE);
 
 	/* copy the list as we're going to reorder it */
@@ -590,7 +604,7 @@ brasero_libisofs_create_volume_thread (gpointer data)
 	grafts = g_slist_sort (grafts, brasero_libisofs_sort_graft_points);
 
 	/* add global exclusions */
-	for (excluded = brasero_track_data_get_excluded (BRASERO_TRACK_DATA (track), FALSE);
+	for (excluded = brasero_track_data_get_excluded_list (BRASERO_TRACK_DATA (track));
 	     excluded; excluded = excluded->next) {
 		gchar *uri, *local;
 
@@ -710,7 +724,7 @@ brasero_libisofs_create_volume_thread (gpointer data)
 							 result);
 					priv->error = g_error_new (BRASERO_BURN_ERROR,
 								   BRASERO_BURN_ERROR_GENERAL,
-								   _("Libisofs reported an error while creating directory \"%s\""),
+								   _("libisofs reported an error while creating directory \"%s\""),
 								   graft->path);
 					g_free (path_name);
 					goto end;
@@ -725,7 +739,7 @@ brasero_libisofs_create_volume_thread (gpointer data)
 							 result);
 					priv->error = g_error_new (BRASERO_BURN_ERROR,
 								   BRASERO_BURN_ERROR_GENERAL,
-								   _("Libisofs reported an error while adding contents to directory \"%s\" (%x)"),
+								   _("libisofs reported an error while adding contents to directory \"%s\" (%x)"),
 								   graft->path,
 								   result);
 					g_free (path_name);
@@ -748,7 +762,7 @@ brasero_libisofs_create_volume_thread (gpointer data)
 							 err);
 					priv->error = g_error_new (BRASERO_BURN_ERROR,
 								   BRASERO_BURN_ERROR_GENERAL,
-								   _("Libisofs reported an error while adding file at path \"%s\""),
+								   _("libisofs reported an error while adding file at path \"%s\""),
 								   graft->path);
 					g_free (path_name);
 					goto end;
@@ -764,7 +778,7 @@ brasero_libisofs_create_volume_thread (gpointer data)
 								 err);
 						priv->error = g_error_new (BRASERO_BURN_ERROR,
 									   BRASERO_BURN_ERROR_GENERAL,
-									   _("Libisofs reported an error while adding file at path \"%s\""),
+									   _("libisofs reported an error while adding file at path \"%s\""),
 									   graft->path);
 						g_free (path_name);
 						goto end;
@@ -777,7 +791,7 @@ brasero_libisofs_create_volume_thread (gpointer data)
 		else if (iso_tree_add_new_dir (ISO_DIR (parent), path_name, NULL) < 0) {
 			priv->error = g_error_new (BRASERO_BURN_ERROR,
 						   BRASERO_BURN_ERROR_GENERAL,
-						   _("Libisofs reported an error while creating directory \"%s\""),
+						   _("libisofs reported an error while creating directory \"%s\""),
 						   graft->path);
 			g_free (path_name);
 			goto end;
@@ -851,7 +865,7 @@ brasero_libisofs_create_volume (BraseroLibisofs *self, GError **error)
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     _("Libisofs could not be initialized."));
+			     _("libisofs could not be initialized."));
 		return BRASERO_BURN_ERR;
 	}
 
@@ -874,6 +888,19 @@ brasero_libisofs_create_volume (BraseroLibisofs *self, GError **error)
 	return BRASERO_BURN_OK;
 }
 
+static void
+brasero_libisofs_clean_output (BraseroLibisofs *self)
+{
+	BraseroLibisofsPrivate *priv;
+
+	priv = BRASERO_LIBISOFS_PRIVATE (self);
+
+	if (priv->libburn_src) {
+		burn_source_free (priv->libburn_src);
+		priv->libburn_src = NULL;
+	}
+}
+
 static BraseroBurnResult
 brasero_libisofs_start (BraseroJob *job,
 			GError **error)
@@ -887,6 +914,10 @@ brasero_libisofs_start (BraseroJob *job,
 
 	brasero_job_get_action (job, &action);
 	if (action == BRASERO_JOB_ACTION_SIZE) {
+		/* do this to avoid a problem when using
+		 * DUMMY flag. libisofs would not generate
+		 * a second time. */
+		brasero_libisofs_clean_output (BRASERO_LIBISOFS (job));
 		brasero_job_set_current_action (BRASERO_JOB (self),
 						BRASERO_BURN_ACTION_GETTING_SIZE,
 						NULL,
@@ -975,22 +1006,6 @@ brasero_libisofs_init (BraseroLibisofs *obj)
 	priv->cond = g_cond_new ();
 }
 
-static void
-brasero_libisofs_clean_output (BraseroLibisofs *self)
-{
-	BraseroLibisofsPrivate *priv;
-
-	priv = BRASERO_LIBISOFS_PRIVATE (self);
-
-	if (priv->libburn_src) {
-		burn_source_free (priv->libburn_src);
-		priv->libburn_src = NULL;
-	}
-
-	/* close libisofs library */
-	iso_finish ();
-}
-
 static void
 brasero_libisofs_finalize (GObject *object)
 {
@@ -1013,18 +1028,21 @@ brasero_libisofs_finalize (GObject *object)
 		priv->cond = NULL;
 	}
 
+	/* close libisofs library */
+	iso_finish ();
+
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static BraseroBurnResult
-brasero_libisofs_export_caps (BraseroPlugin *plugin, gchar **error)
+static void
+brasero_libisofs_export_caps (BraseroPlugin *plugin)
 {
 	GSList *output;
 	GSList *input;
 
 	brasero_plugin_define (plugin,
 			       "libisofs",
-			       _("Libisofs creates disc images from files"),
+			       _("Creates disc images from a file selection"),
 			       "Philippe Rouquier",
 			       0);
 
@@ -1073,6 +1091,4 @@ brasero_libisofs_export_caps (BraseroPlugin *plugin, gchar **error)
 	g_slist_free (output);
 
 	brasero_plugin_register_group (plugin, _(LIBBURNIA_DESCRIPTION));
-
-	return BRASERO_BURN_OK;
 }
